/*
 * Copyright (c) 2009-2010 by Oleg Trifonov <otrifonow@gmail.com>
 *
 * http://trol.0fees.net
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU General Public License version 2
 * or the GNU Lesser General Public License version 2.1, both as
 * published by the Free Software Foundation.
 */


#ifndef _CONFIG_H_
#define _CONFIG_H_

#include <string>
#include <vector>

class Config {
public:
	Config();
	~Config();

	bool Load();

	bool SelectDevice(const std::string &id);

	bool ParseCommandLine(int argc, char *argv[]);

	// loader settings
	std::string port;			// com-port name
	int baudrate;				// com-port baudrate
	bool verify;				// verify data after writing
	bool verbose;				// verbose output
	bool quellprogress;		// don't show the progress indicator
	bool smart;					// "smart" writing mode
	std::string device;		// microcontroller name
	bool no_write;				// don't write anything to chip, debug mode only

	// device settings
	std::string device_id;			// device name
	std::string device_desc;		// device description
	int device_rom_size;				// size of ROM
	int device_page_size;			// size of ROM page
	int device_eeprom_size;			// szie of EEPROM

	struct TOperation {
		char type;		// operation type: r|w|v
		char target;	// memory type: f|e
		std::string filename;
	};

	std::vector<TOperation*> operations;

private:
	struct TDeviceInfo {
		std::string id;
		std::string desc;
		int rom_size;
		int page_size;
		int eeprom_size;
	};

	std::vector<TDeviceInfo*> devices;
	int line;							// current line in file
	std::string fileName;			// filename

	bool InitBoolParam(bool &val, const std::string &str);
	bool InitIntParam(int &val, const std::string &str);
	bool InitStrParam(std::string &val, const std::string &str);
};

#endif