/*
 * Copyright (c) 2009-2010 by Oleg Trifonov <otrifonow@gmail.com>
 *
 * http://trol.0fees.net
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU General Public License version 2
 * or the GNU Lesser General Public License version 2.1, both as
 * published by the Free Software Foundation.
 */


#ifndef _DEVICE_H_
#define _DEVICE_H_

#include "RS232.h"

class Device {
public:
	Device();
	~Device();

	// Connect to the bootloader
	bool Open(const char *comName, int baudRate);

	// Reads a memory page
	bool ReadPage(int offset, unsigned char *data, int size);

	// Reads all chip memory
	bool ReadAll(unsigned char *data, int size);

	// Writes a memory page
	bool WritePage(int offset, unsigned char *data, int size);

	// Writes all memory pages
	// If smart == true, then before writing of a page it will be read completely.
	// And if changes is not required, the page won't write
	// If byte value is less than zero, this byte won't changed
	bool WriteAll(int *data, int size, bool smart);

	// Jump to program offset
	bool Jump(int offset);

	// Reset the bootloader and verify the signature
	bool Reset();

	// Gets a bootloader start offset
	int GetLoaderOffset();

	void SetPageSize(int size) { pageSize = size; }

	void SetOutputLevel(bool aVerbose, bool aQuellProgress) { verbose = aVerbose; quellprogress = aQuellProgress; };

	// Disable writing to chip memory (debug mode)
	void DisableWriting() { noWrite = true; }

	int pagesWrited;			// WriteAll() stores here the count of writed pages
	int pagesWritedTotal;	// WriteAll() stores here the count of pages, whitch were supposed to been written down
private:
	RS232 com;

	int pageSize;
	bool verbose;
	bool quellprogress;
	bool noWrite;
	

	// Set the Z-register
	bool CommandZ(int z);

	// Reads a data block
	bool CommandR(unsigned char *data, int size);

	// Writes a data block
	bool CommandW(unsigned char *data, int size);

	// Reads the R0:R1 pair, then one byte into SPMCR and executes the SPM instruction
	bool CommandP(int r0r1, int spmcr);

	// Sends one byte in hex format
	void sendHexByte(unsigned char b);

	// Sends one word in hex format
	void sendHexWord(unsigned short w);

	// Reads one byte in hex format. returns -1 if there was an error
	int getHexByte();

	// Reads one word in hex format. returns -1 if there was an error
	int getHexWord();

	// Prints the progressbar
	void Progress(const char* msg, int percent, float time);
};

#endif 