
#ifndef _RS232_H_

#define _RS232_H_

#ifdef WIN32
	#include <windows.h>
#else
	#include <termios.h>
#endif


class RS232 {
public:
	RS232(const char *name, unsigned int baud, bool parity = false, unsigned char stopBits = 0);
	RS232();
	~RS232();

	bool Open(const char *name, unsigned int baud, bool parity = false, unsigned char stopBits = 0);

	unsigned int Read(void *buf, unsigned int size);

	unsigned int Write(void *buf, unsigned int size);

	void Close();

	// Set/reset the DTR line
	bool SetDTR(bool val);
	
	// Set/reset the RTS line
	bool SetRTS(bool val);
	
	void WriteByte(unsigned int val);

	unsigned int ReadByte();
	
	bool IsOk() { return !iError; }
	
	// Set reading timeout (uSec)
	void SetReadTimeOut(int to) { readTimeout = to; };
	
#ifdef __linux__
	struct termios new_port_settings, old_port_settings;
	static int getBaudrateValue(int baudrate);
#endif

private:
	bool iError;
	int readTimeout;			// reading operation timout, uSec
#ifdef WIN32
	HANDLE handle;				// port file descriptor
#else
	int handle;
#endif

};

#endif // _RS232_H_